#ifndef NODE_HPP
#define NODE_HPP

#include <vector>

using Vector = std::vector<double>;
using Matrix = std::vector<Vector>;
using List = std::vector<unsigned>;

class Node {
public:
    // Default constructor creates a leaf node
    Node() = default;
    // Whoever creates the node is responsible for deciding which particles are in which octant and for allocating the localParticles array 
    // Node will write to localParticles array
    Node(double mass, double* positions, unsigned nParticles, double softening, unsigned depth, 
        double size, Vector center, List localParticles, unsigned nLocalParticles);
    // Destuctor
    ~Node();
    // Copy constructor (default)
    Node(const Node&) = default;
    // Copy assignment
    Node& operator=(const Node&);
    // Update node: calculate center of mass, get local particles 
    void update(List particles, unsigned nParticles);
    // Calculate force on a particle recursively
    Vector calculateForce(unsigned particle);
    // Calculate the matrix Q
    Vector multipole(Vector y, double ynorm) const;
    // Update center of mass and total mass in the node
    void updateMassAndCOM();
    // Function that takes a particle index in the global particles_ array and returns the octant it is in relative to the center fo the current node 
    // this function doesn't check if the particle is actually in the node.
    unsigned getOctant(unsigned particle);
    // get center of mass
    Vector getCOM() const;

private:
    // Node* root_;
    // Node* parent_;
    Node* children_[8];

    double mass_;
    double* positions_;
    unsigned nParticles_;
    double softening_;

    unsigned depth_;
    double size_;
    Vector center_;
    Vector centerOfMass_;

    // Contains the indices of the particles that are in the children of this node for the particles_ array
    List childParticles_[8];
    unsigned nLocalParticles_;

    double totalMass_;
    const double theta0_ = 0.0;
    const double minNParticles_ = 32;
    const double maxDepth_ = 24;
    bool isLeaf_;
};

#endif /* NODE_HPP */